#include "attributewindow.h"




AttributeWindow::AttributeWindow(Defaults *defaults)
 : BC_Window("", MEGREY, "Split 2000: Attributes", 430, 130, 0, 0)
{
	this->defaults = defaults;
}

AttributeWindow::~AttributeWindow()
{
	delete ok;
	delete cancel;
	delete segmentbox;
	delete channelsbox;
	delete bitsitems[0];
	delete bitsitems[1];
	delete bitsitems[2];
	delete bitsmenu;
	delete keepbox;
}

int AttributeWindow::create_objects()
{
	load_defaults();
	char string[1024];

	add_tool(new BC_Title(5, 5, "Select the attributes of these files:"));
	add_tool(ok = new SplitOkButton(this));
	add_tool(cancel = new SplitCancelButton(this));
	sprintf(string, "%d", segment_size);
	add_tool(new BC_Title(140, 30, "Segment size:"));
	add_tool(segmentbox = new SplitSegmentSize(this, string));
	sprintf(string, "%d", channels);
	add_tool(new BC_Title(5, 30, "Channels:"));
	add_tool(channelsbox = new SplitChannels(this, string));
	sprintf(string, "%d", bits);
	add_tool(new BC_Title(330, 30, "Bits:"));
	add_tool(bitsmenu = new SplitBits(this, string));
	bitsmenu->add_item(bitsitems[0] = new SplitBitsItem(bitsmenu, "8"));
	bitsmenu->add_item(bitsitems[1] = new SplitBitsItem(bitsmenu, "16"));
	bitsmenu->add_item(bitsitems[2] = new SplitBitsItem(bitsmenu, "24"));
	add_tool(new BC_Title(200, 60, "Keep original:"));
	add_tool(keepbox = new SplitKeepOriginal(this, keep_original));
}

int AttributeWindow::load_defaults()
{
	channels = defaults->get("CHANNELS", 2);
	bits = defaults->get("BITS", 16);
	keep_original = defaults->get("KEEP", 0);
	segment_size = defaults->get("SEGMENT", 27182);
}


int AttributeWindow::save_defaults()
{
	defaults->update("CHANNELS", channels);
	defaults->update("BITS", bits);
	defaults->update("KEEP", keep_original);
	defaults->update("SEGMENT", segment_size);
}

SplitOkButton::SplitOkButton(AttributeWindow *dialog)
 : BC_BigButton(10, 90, "OK")
{
	this->dialog = dialog;
}

SplitOkButton::~SplitOkButton()
{
}

int SplitOkButton::handle_event()
{
	dialog->set_done(0);
}

int SplitOkButton::keypress_event()
{
	return 0;
}

SplitCancelButton::SplitCancelButton(AttributeWindow *dialog)
 : BC_BigButton(320, 90, "Cancel")
{
	this->dialog = dialog;
}

SplitCancelButton::~SplitCancelButton()
{
}
	
int SplitCancelButton::handle_event()
{
	dialog->set_done(1);
}

int SplitCancelButton::keypress_event()
{
	return 0;
}

SplitChannels::SplitChannels(AttributeWindow *dialog, char *text)
 : BC_TextBox(80, 25, 50, text)
{
	this->dialog = dialog;
}

SplitChannels::~SplitChannels()
{
}
	
int SplitChannels::handle_event()
{
	dialog->channels = atol(get_text());
}

SplitSegmentSize::SplitSegmentSize(AttributeWindow *dialog, char *text)
 : BC_TextBox(240, 25, 75, text)
{
	this->dialog = dialog;
}

SplitSegmentSize::~SplitSegmentSize()
{
}
	
int SplitSegmentSize::handle_event()
{
	dialog->segment_size = atol(get_text());
}

SplitBits::SplitBits(AttributeWindow *dialog, char *text)
 : BC_PopupMenu(370, 25, 50, text)
{
	this->dialog = dialog;
}

SplitBits::~SplitBits()
{
}
	
int SplitBits::handle_event()
{
	dialog->bits = atol(text);
}

SplitKeepOriginal::SplitKeepOriginal(AttributeWindow *dialog, int value)
 : BC_CheckBox(300, 60, 15, 15, value)
{
	this->dialog = dialog;
}

SplitKeepOriginal::~SplitKeepOriginal()
{
}
	
int SplitKeepOriginal::handle_event()
{
	dialog->keep_original = down;
}

SplitBitsItem::SplitBitsItem(SplitBits *bits, char *text)
 : BC_PopupItem(text)
{
	this->bits = bits;
}

SplitBitsItem::~SplitBitsItem()
{
}
	
int SplitBitsItem::handle_event()
{
	get_menu()->update(text);
	get_menu()->handle_event();
}
