#include "batchwindow.h"

BatchSubWindow::BatchSubWindow(BC_Window *window, 
					char *init_directory, 
					char *caption1, 
					char *caption2, 
					ArrayList<BatchItem*> *batch, 
					int x, 
					int y)
 : BC_SubWindow(x, y, window->get_w(), 385, MEGREY)
{
	strcpy(directory, init_directory);   // save initial directory
	
	strcpy(this->caption1, caption1);
	strcpy(this->caption2, caption2);
	dir = new FileSystem;
	this->batch = batch;
}

BatchSubWindow::~BatchSubWindow()
{
	delete dir;
	delete intext;
	delete directory_title;
	delete inlist;
	delete outlist;
}

int BatchSubWindow::create_objects()
{
	create_dirlist(directory);
	add_border(LTGREY, MEGREY, DKGREY);
	add_tool(new BC_Title(10, 5, caption1));
	add_tool(new BC_Title(320, 5, caption2));

	add_tool(new BatchAdd(this));
	add_tool(new BatchDelete(this));
	add_tool(intext = new InText(this, ""));
	add_tool(directory_title = new BatchDirectory(this, directory));
	add_tool(inlist = new InList(this, &dir_list));
	add_tool(outlist = new OutList(this, (ArrayList <BC_ListBoxItem*> *)batch));
	outlist->stay_highlighted();
	inlist->activate();
}

int BatchSubWindow::create_dirlist(char *text)
{
	dir->complete_path(text);
	dir->update(text);
	dir_list.remove_all();
	for(int i = 0; i < dir->dir_list.total; i++)
	{
		dir_list.append(new BC_ListBoxItem(dir->dir_list.values[i]->name,
					dir->dir_list.values[i]->is_dir ? BLUE : BLACK));
	}
}

int BatchSubWindow::resize_event(int w, int h)
{
}

int BatchSubWindow::set_filter(char *filter)
{
	dir->set_filter(filter);
}

int BatchSubWindow::submit_file(char *text)
{
	if(!dir->is_dir(text))
	{            // is a directory
		create_dirlist(text);     // change to it
		directory_title->update(text);
		dir->extract_dir(directory, text);
		inlist->set_contents(&dir_list, 0, 1);
		intext->update("");
		inlist->reset_query();
		return 1;
	}
	else
	{            // not a directory, load it now
		dir->extract_dir(directory, text);     // directory is saved in directory for default updating
		dir->complete_path(text);

// add text to batch here
		char name[1024];
		dir->extract_name(name, text);
		for(int i = 0; i < batch->total; i++)
		{
			if(!strcmp(batch->values[i]->text, name))
			{           // exists
				return 1;
			}
		}
		batch->append(new BatchItem(text, name));

		inlist->set_selection(-1);
		inlist->reset_query();
		intext->update("");
		outlist->set_contents((ArrayList<BC_ListBoxItem*> *)batch, 0, 1);
		outlist->reset_query();
	}
	return 0;
}

int BatchSubWindow::delete_file(char *text)
{
	int result = 0;
	for(int i = 0; i < batch->total && !result; i++)
	{
		if(!strcmp(batch->values[i]->text, text))
		{
			delete batch->values[i];
			batch->remove_number(i);
			result = 1;
		}
	}

	outlist->set_contents((ArrayList<BC_ListBoxItem*> *)batch, 0, 1);
	outlist->reset_query();
	return 0;
}

BatchAdd::BatchAdd(BatchSubWindow *subwindow)
 : BC_BigButton(10, 350, "Select")
{ this->subwindow = subwindow; }

int BatchAdd::handle_event()
{
	subwindow->submit_file(subwindow->intext->get_text());
}

int BatchAdd::keypress_event()
{
	if(get_keypress() == 13) { handle_event(); return 1; }
	return 0;
}

BatchDelete::BatchDelete(BatchSubWindow *subwindow) 
 : BC_BigButton(320, 350, "Remove from batch")
{ this->subwindow = subwindow; }

int BatchDelete::handle_event()
{
	char string[1024];
	subwindow->outlist->get_selection(string);
	//printf("batch delete\n");
	subwindow->delete_file(string);
}

InText::InText(BatchSubWindow *subwindow, char *text)
 : BC_TextBox(10, 320, 300, "")
{ this->subwindow = subwindow; }

int InText::handle_event()
{
}

BatchDirectory::BatchDirectory(BatchSubWindow *subwindow, char *text)
 : BC_Title(10, 300, text)
{
}

InList::InList(BatchSubWindow *subwindow, ArrayList <BC_ListBoxItem*> *dir_list)
 : BC_ListBox(10, 30, 250, 250, dir_list, 0, 1)
{ this->subwindow = subwindow; }

InList::~InList() {}

int InList::handle_event()
{
	char string[1024];
	get_selection(string);
	
	subwindow->submit_file(string);
}

int InList::selection_changed()
{
	char string[1024];
	get_selection(string);
	subwindow->intext->update(string);
}

OutList::OutList(BatchSubWindow *subwindow, ArrayList <BC_ListBoxItem*> *batch)
 : BC_ListBox(300, 30, 200, 250, batch, 0, 1)
{ this->subwindow = subwindow; }

OutList::~OutList() {}

int OutList::handle_event()
{
	char string[1024];
	get_selection(string);
	
	subwindow->delete_file(string);
}

int OutList::selection_changed()
{
}
