#include "filesystem.h"
#include "joinengine.h"
#include <unistd.h>


// algorithm all multichannel and single channel > 8 bits:
// [1st byte[channel1 segment][channel2 segment][channel1 segment][channel2 segment]][2nd byte[channel1 segment][channel2 segment]...

// algorithm 8 bits mono:
// [1st 4 bits[channel1 segment][channel2 segment][channel1 segment]...][last 4 bits[channel1 segment][channel2 segment]..]

JoinEngine::JoinEngine(int keep_original, 
		ArrayList <BatchItem*> *batch, 
		char *decompressor, 
		char *output_directory)
{
	this->keep_original = keep_original;
	this->batch = batch;
	this->decompressor = decompressor;
	strcpy(this->output_directory, output_directory);
}

JoinEngine::~JoinEngine()
{
}

long JoinEngine::get_number(FILE *in)
{
	long number = 0, byte = 0;

	byte = fgetc(in);
	byte <<= 24;
	number = byte & 0xFF000000;
	byte = fgetc(in);
	byte <<= 16;
	number += byte & 0xFF0000;
	byte = fgetc(in);
	byte <<= 8;
	number += byte & 0xFF00;
	number += byte = fgetc(in);
	return number;
}

int JoinEngine::update_user(char *string)
{        // stdout is the output file so use stderr
	if(interactive) progress->update(string);
	else
	{
		fprintf(stderr, "%s", string);
		fflush(stderr);
	}
}

int JoinEngine::run(int interactive)
{
// set up display
	char string[1024];
	char text[1024];
	char filename_in_text[1024], filename_out_text[1024];
	FileSystem fs;
	int result = 0;
	int i;
	int j, k, result2;
	long total_in;
	long buffer_out_segment;
	register int frame;
	long buffer_in_total, buffer_out_total;
	char *buffer_out;
	char *buffer_in;

	sprintf(text, "");

	this->interactive = interactive;

	if(interactive)
	{
		progress = new Progress("Decompressor", text);
		progress->create_objects();
		progress->start();
	}

	for(i = 0; i < batch->total && result != 1; i++)
	{
		FILE *in, *out;
		FILE *test_alias;
		char filename_in[1024], filename_out[1024], exec_script[1024];

		result = 0;

// ========================== determine filenames
		strcpy(filename_in, batch->values[i]->path);
		fs.extract_name(filename_in_text, filename_in);
		fs.extract_name(filename_out_text, filename_in);

// remove .spl
		for(j = 0, k = strlen(filename_out_text) - 1, result2 = 0; !result2 && k > 0; j++, k--)
		{
			if(filename_out_text[k] == '.') result2 = 1;
			filename_out_text[k] = 0;
		}

		fs.join_names(filename_out, output_directory, filename_out_text);

//printf("%s %s %s\n", filename_out, output_directory, filename_out_text);

		if(!result2)
		{
			sprintf(string, "%s doesn't end in .spl.  Skipping.\n", filename_in);
			update_user(string);
			result = 2;
		}

// set up the decompressor

		sprintf(exec_script, "%s", decompressor);

// =============================== test existence of output file
		if(!result && (in = fopen(filename_out, "rb")))
		{
			sprintf(string, "%s already exists.  Skipping.\n", filename_out);
			update_user(string);
			result = 2;
			fclose(in);
		}

// ============================= open the input file
		if(!result && !(test_alias = fopen(filename_in, "rb")))
		{
			sprintf(string, "Couldn't open %s.\n", filename_in);
			update_user(string);
			result = 1;
		}

 		if(!result) dup2(fileno(test_alias), 0);

		if(!result && !(in = popen(exec_script, "rb")))
		{
			sprintf(string, "Couldn't fork decompressor.\n");
			update_user(string);
			fclose(test_alias);
			result = 1;
		}

// ============================= check magic number
		if(!result)
		{
			long magic_number = get_number(in);

			if(magic_number != 7374)
			{
				result = 1;
				fclose(in);
				fclose(test_alias);
				sprintf(string, "%s has wrong magic number!\n", filename_in);
				update_user(string);
			}
		}

// open the output file
		if(!result && !(out = fopen(filename_out, "w+b")))
		{
			sprintf(string, "Couldn't open %s.\n", filename_out);
			update_user(string);
			result = 1;
			fclose(in);
		}

		if(!result)
		{
			int segment = get_number(in);
			int channels = get_number(in); // channels
			int byte;
			int bits = get_number(in); // bits
			int bytes = bits / 8;
			long length_byte_strip = get_number(in);  // byte strip
			long length_out = get_number(in);          // length of real file
			get_number(in);       // padding
			get_number(in);       // padding
			get_number(in);       // padding
			get_number(in);       // padding
			get_number(in);       // padding

			total_in = length_byte_strip * bytes;
			if(bits == 8 && channels == 1) total_in = length_byte_strip * 2;
			frame = bytes * channels;
			buffer_out_segment = segment * bytes * channels;
			if(bits == 8 && channels == 1)
			{
				segment /= 2;       // segment must be divisible by nibbles
				segment *= 2;
				buffer_out_segment = segment * 2 * channels;
			}

			buffer_out_total = (int)(1000000 / buffer_out_segment) * buffer_out_segment;
			//while(buffer_out_total < 1000000)
			//{
// Egcs math error
//printf("JoinEngine::run buffer_out_total %d + buffer_out_segment %d = %d\n", buffer_out_total, buffer_out_segment, buffer_out_total + buffer_out_segment);
			//	buffer_out_total = buffer_out_total + buffer_out_segment;
			//}

			buffer_out = new char[buffer_out_total + 1];

			buffer_in_total = buffer_out_total / bytes;
			if(bits == 8 && channels == 1) buffer_in_total = buffer_out_total / 2;
			buffer_in = new char[buffer_in_total];
//printf("JoinEngine::run buffer_out_segment %d buffer_out_total %d buffer_in_total %d\n", buffer_out_segment, buffer_out_total, buffer_in_total);

// display some information
			sprintf(string, "%s -> %s\n", filename_in_text, filename_out_text);
			update_user(string);

			sprintf(string, "%d channels %d bits %d segment size\n", channels, bits, segment);
			update_user(string);
			sprintf(string, "Using: %s\n", decompressor);
			update_user(string);
			if(keep_original) sprintf(string, "Keeping source.\n");
			else sprintf(string, "Deleting source.\n");
			update_user(string);

			if(interactive) progress->update_length(total_in);


// ===================================== 8 bits
			if(bits == 8 && channels == 1)
			{
				int input_nibble;
				long buffer_in_length;
				long position_in;
				long reread_length;
				long channel_pointer_out[channels];
				long channel_pointer_in[channels];
				register int segment_pointer;
				register int channel;
				register char nibble;
				long total_result;
				for(input_nibble = 0; input_nibble < 2 && !result; input_nibble++)
				{
					if(input_nibble > 0) fseek(out, 0, SEEK_SET);
					buffer_in_length = buffer_in_total;
					for(position_in = 0; position_in < length_byte_strip && !result;)
					{
						if(length_byte_strip - position_in < buffer_in_length)
						{
							buffer_in_length = length_byte_strip - position_in;
							for(j = 0; j < buffer_in_total;) buffer_in[j++] = 0;
						}
						if(input_nibble > 0)
						{
							reread_length = buffer_in_length * 2;
							if(length_out - ftell(out) < reread_length) reread_length = length_out - ftell(out);
							fread(buffer_out, reread_length, 1, out);
							fseek(out, -reread_length, SEEK_CUR);
						}
// read a buffer length
						if(!fread(buffer_in, buffer_in_length, 1, in))
						{
							result = 1;
							sprintf(string, "Read failed.\n");
							update_user(string);
						}
						else
						{
// set up the channel pointers for the output buffer
							for(j = 0; j < channels; j++)
							{
								channel_pointer_out[j] = j;
							}

// set up the channel pointers for the intput buffer
							for(j = 0; j < channels; j++)
							{
								channel_pointer_in[j] = j * segment;
							}

// ================ join the input buffer into the output buffer
							while(channel_pointer_in[0] < buffer_in_length)
							{
// join segments for all channels simultaneously
// ========================== high input nibble
								if(input_nibble == 0)
								{
									for(segment_pointer = 0; segment_pointer < segment; segment_pointer++)
									{
// iterate through each channel for each segment
										for(channel = 0; channel < channels; channel++)
										{
// get high nibble from segment
											nibble = buffer_in[channel_pointer_in[channel]];
											nibble &= 0xF0;
											buffer_out[channel_pointer_out[channel]] = nibble;
											channel_pointer_out[channel] += frame;

// get low nibble from segment
											nibble = buffer_in[channel_pointer_in[channel]];
											nibble <<= 4;
											nibble &= 0xF0;
											buffer_out[channel_pointer_out[channel]] = nibble;
											channel_pointer_out[channel] += frame;
											channel_pointer_in[channel]++;
										}
									}
// realign channel pointers for next segment
									for(channel = 0; channel < channels; channel++)
									{
										channel_pointer_in[channel] += segment * channels - segment;
									}
								}
								else
								{
// ================================== low input nibble
									for(segment_pointer = 0; segment_pointer < segment; segment_pointer++)
									{
// iterate through each channel for each segment
										for(channel = 0; channel < channels; channel++)
										{
// get high nibble from segment
											nibble = buffer_in[channel_pointer_in[channel]];
											nibble >>= 4;
											nibble &= 0xF;
											buffer_out[channel_pointer_out[channel]] += nibble;
											channel_pointer_out[channel] += frame;

// get low nibble from segment
											nibble = buffer_in[channel_pointer_in[channel]];
											nibble &= 0x0F;
											buffer_out[channel_pointer_out[channel]] += nibble;
											channel_pointer_out[channel] += frame;
											channel_pointer_in[channel]++;
										}
									}
// realign channel pointers for next segment
									for(channel = 0; channel < channels; channel++)
									{
										channel_pointer_in[channel] += segment * channels - segment;
									}
								}
							}

// enforce real length of output file
							if(length_out - ftell(out) < channel_pointer_out[0])
							{
								channel_pointer_out[0] = length_out - ftell(out);
							}

							if(!fwrite(buffer_out, channel_pointer_out[0], 1, out))
							{
								sprintf(string, "Write failed.\n");
								update_user(string);
								result = 1;
							}
							position_in += channel_pointer_in[0];
// update progress

// fake actual file length
							total_result = length_byte_strip * input_nibble + position_in;
							if(total_result > length_out) total_result = length_out;

							if(!interactive)
							{
								sprintf(string, "%ld bytes joined\r", total_result);
								update_user(string);
							}
							else
							{
								if(progress->update_position(total_result)) result = 1;
							}
						}
					}
				}   // next nibble
			}
			else

// ===================================== 16 bits

			for(byte = 0; byte < bytes && !result; byte++)
			{
				long buffer_in_length;
				long position_in;
				long reread_length;
				long channel_pointer_out[channels];
				long channel_pointer_in[channels];
				register int segment_pointer;
				register int channel;
				long total_result;
// read each byte from the file

// reread buffers for additional bytes
				if(byte > 0)

// relocate output to temporary file and read from temporary file in DOS
				{
					fseek(out, byte, SEEK_SET);
				}


				buffer_in_length = buffer_in_total;
				for(position_in = 0; position_in < length_byte_strip && !result;)
				{

					if(length_byte_strip - position_in < buffer_in_length)
					{
						buffer_in_length = length_byte_strip - position_in;
						for(j = 0; j < buffer_in_total;) buffer_in[j++] = 0;
					}

// read previous buffer back for striping
					if(byte > 0)
					{
						int reread_result;
						long reread_position;
						reread_length = buffer_in_length * bytes;

						if(length_out - ftell(out) < reread_length) reread_length = length_out - ftell(out);

						reread_position = ftell(out);
						reread_result = fread(buffer_out, reread_length, 1, out);
						fseek(out, reread_position, SEEK_SET);
						if(!reread_result)
						{
							sprintf(string, "*** Reread failed.\n");
							update_user(string);
							result = 1;
						}
					}

// read a buffer length
					if(!fread(buffer_in, buffer_in_length, 1, in))
					{
						result = 1;
						sprintf(string, "*** Read failed.\n");
						update_user(string);
					}
					else
					{
// set up the channel pointers for the output buffer
						for(j = 0; j < channels; j++)
						{
							channel_pointer_out[j] = j * bytes;
						}

// set up the channel pointers for the intput buffer
						for(j = 0; j < channels; j++)
						{
							channel_pointer_in[j] = j * segment;
						}
// join the input buffer into the output buffer

						while(channel_pointer_in[0] < buffer_in_length)
						{
// join segments for all channels simultaneously
//printf("JoinEngine::run 12 channel_pointer_in[0] %d buffer_in_length %d segment %d\n", channel_pointer_in[0], buffer_in_length, segment);
							for(segment_pointer = 0; segment_pointer < segment; segment_pointer++)
							{
// iterate through each channel for each segment
								for(channel = 0; channel < channels; channel++)
								{
									buffer_out[channel_pointer_out[channel]] = buffer_in[channel_pointer_in[channel]];
									channel_pointer_in[channel]++;
									channel_pointer_out[channel] += frame;
								}
							}
// realign channel pointers for next segment
//printf("JoinEngine::run 15\n");
							for(channel = 0; channel < channels; channel++)
							{
								channel_pointer_in[channel] += segment * channels - segment;
							}
						}

// enforce real length of output file
						if(length_out - ftell(out) < channel_pointer_out[0])
						{
							channel_pointer_out[0] = length_out - ftell(out);
						}

						if(!fwrite(buffer_out, channel_pointer_out[0], 1, out))
						{
							sprintf(string, "Write failed.\n");
							update_user(string);
							result = 1;
						}
						position_in += channel_pointer_in[0];

// update progress
// fake actual file length
						total_result = length_byte_strip * byte + position_in;
						if(total_result > length_out) total_result = length_out;

						if(!interactive)
						{
							fprintf(stderr, "%ld bytes joined\r", total_result);
							fflush(stderr);
						}
						else
						{
							if(progress->update_position(total_result)) result = 1;
						}
					}
				}
			}

			delete buffer_in;
			delete buffer_out;

			fclose(in);
			fclose(out);
			fclose(test_alias);

// remove original or output if error
			if(!keep_original && !result) remove(filename_in);
			if(result == 1) remove(filename_out);
			if(!result)
			{
// add eoln
				sprintf(string, "\n");
				update_user(string);
			}
		}
	}

	if(interactive)
	{
		//progress->end(); // prevents core dump
		progress->update("Done\n");
		progress->update_button("OK");
		progress->window->cancelled_ = 2;
		//progress->run();
		progress->join();
		delete progress;
	}
}
