#include "attributewindow.h"
#include "joinengine.h"
#include "mainwindow.h"
#include "outputwindow.h"
#include "splitengine.h"

main(int argc, char *argv[])
{
	printf("Split2000 (c) 1998 Adam Williams\n");
	printf("No warranties of any kind.\n");

	ArrayList<BatchItem*> batch;

	int i;
	int interactive = 1;
	int channels = 2;
	int keep_original = 0;
	int bits = 16;
	int decompress = 0;
	char compressor[1024];
	char decompressor[1024];
	char output_directory[1024];
	int segment = 27182;

	sprintf(compressor, "bzip2");
	sprintf(decompressor, "bzip2 -d");
	sprintf(output_directory, "");

	if(!strcmp(argv[0], "join2000")) { decompress = 1; interactive = 0; }
	for(i = 1; i < argc; i++)
	{
		if(!strcmp(argv[i], "-h"))
		{
			printf("Split2000 options:\n");
			printf("-d               decompress\n");
			printf("-c <number>      channels (2)\n");
			printf("-k               keep original (NO)\n");
			printf("-b <number>      bits: 8, 16, or 24 (16)\n");
			printf("-u <command>     use <command> as compressor (bzip2)\n");
			printf("-s <number>      segment size (27182)\n");
			printf("Example:\n");
			printf("split2000 -c 1 -k -b 8 -s 27182 -u bzip2 test.wav\n");
			printf("      compress using bzip2 segment size 27182 and keep original.\n");
			printf("split2000 -c 1 -k -b 8 test.wav\n");
			printf("      compress the 1 channel, 8 bit test.wav and keep original.\n");
			exit(0);
		}
		else
		if(!strcmp(argv[i], "-d"))
		{
			decompress = 1;
			interactive = 0;
		}
		else
		if(!strcmp(argv[i], "-c"))
		{
			channels = atol(argv[++i]);
			interactive = 0;
		}
		else
		if(!strcmp(argv[i], "-k"))
		{
			keep_original = 1;
			interactive = 0;
		}
		else
		if(!strcmp(argv[i], "-b"))
		{
			bits = atol(argv[++i]);
			interactive = 0;
			if(bits != 8 && bits != 16 && bits != 24)
			{
				printf("Invalid number of bits.\n");
				exit(1);
			}
		}
		else
		if(!strcmp(argv[i], "-s"))
		{
			segment = atol(argv[++i]);
			interactive = 0;
		}
		else
		if(!strcmp(argv[i], "-u"))
		{
			strcpy(compressor, argv[++i]);
			strcpy(decompressor, argv[i]);
		}
		else
		if(argv[i][0] == '-')
		{
			printf("unknown option: %s\n", argv[i]);
			exit(1);
		}
		else
		{
			batch.append(new BatchItem(argv[i], argv[i]));
			interactive = 0;
		}
	}

	if(interactive)
	{
// read defaults
		Defaults defaults("~/.splitrc");
		defaults.load();

		int result1 = 0, result2 = 0, result3 = 0;
		do{
			result1 = result2 = result3 = 0;

			{
				MainWindow mwindow(&batch, &defaults);
				mwindow.create_objects();
				result1 = mwindow.run_window();
				mwindow.save_defaults();
				strcpy(compressor, mwindow.compressor);
				strcpy(decompressor, mwindow.decompressor);
				keep_original = mwindow.keep_original;
				
				//if(result1 != 1) mwindow.get_batch(batch, total_files);
			}

			if(result1 == 0)
			{
// get attributes of batch files for compression
				{
					AttributeWindow attributes(&defaults);
					attributes.create_objects();
					result2 = attributes.run_window();
					attributes.save_defaults();
					channels = attributes.channels;
					bits = attributes.bits;
					segment = attributes.segment_size;
					keep_original = attributes.keep_original;
				}

				if(!result2)
				{
// get the output directory for files
					defaults.get("OUTDIRECTORY", output_directory);
					
					{
						OutputWindow location(&defaults, output_directory);
						location.create_objects();
						result3 = location.run_window();

						strcpy(output_directory, location.get_directory());
					}

					defaults.update("OUTDIRECTORY", output_directory);

					if(!result3)
					{
// perform compression
						SplitEngine engine(channels, 
							keep_original, 
							bits, 
							segment, 
							&batch, 
							compressor, 
							output_directory);
						engine.run(interactive);
					}
				}
			}
			else
			if(result1 == 2)
			{
// decompress
// get the output directory for files
					defaults.get("OUTDIRECTORY", output_directory);

					{
						OutputWindow location(&defaults, output_directory);
						location.create_objects();
						result3 = location.run_window();

						strcpy(output_directory, location.get_directory());
					}

					defaults.update("OUTDIRECTORY", output_directory);

					if(!result3)
					{
// perform decompression
						JoinEngine engine(keep_original, 
							&batch, 
							decompressor, 
							output_directory);
						engine.run(interactive);
					}
			}
		}while(result1 != 1 && (result2 || result3));
		
		defaults.save();
	}
	else
	if(decompress)
	{
		JoinEngine engine(keep_original, 
			&batch, 
			decompressor, 
			output_directory);
		engine.run(interactive);
	}
	else
	{
		SplitEngine engine(channels, 
			keep_original, 
			bits, 
			segment, 
			&batch, 
			compressor, 
			output_directory);
		engine.run(interactive);
	}
}
