#include "mainmenu.h"
#include "mainwindow.h"

MainWindow::MainWindow(ArrayList<BatchItem*> *batch, Defaults *defaults)
 : BC_Window("", MEGREY, "Split 2000", 530, 420, 0, 0)
{
	this->defaults = defaults;
	this->batch = batch;
}

MainWindow::~MainWindow()
{
	delete batchwindow;
	delete compressbutton;
	delete uncompressbutton;
	delete quitbutton;
	delete sw;
	delete prefs;
}

int MainWindow::create_objects()
{
	load_defaults();

	add_subwindow(sw = new BC_SubWindow(0, 0, get_w(), 35, MECYAN));
	sw->add_border(LTCYAN, MECYAN, DKCYAN);
	sw->add_tool(compressbutton = new CompressButton(this));
	sw->add_tool(uncompressbutton = new UnCompressButton(this));
	sw->add_tool(quitbutton = new QuitButton(this));
	sw->add_tool(prefsbutton = new PreferencesButton (this));

	char directory[1024];
	sprintf(directory, "~");
	defaults->get("DIRECTORY", directory);

	add_subwindow(batchwindow = new BatchSubWindow(this, directory, "Files", "Batch", batch, 0, 35));
	prefs = new PreferencesThread(this);

}

int MainWindow::load_defaults()
{
	keep_original = defaults->get("KEEP", 0);
	sprintf(compressor, "bzip2");
	sprintf(decompressor, "bzip2 -d");
	defaults->get("COMPRESSOR", compressor);
	defaults->get("DECOMPRESSOR", decompressor);
}

int MainWindow::save_defaults()
{
	defaults->update("DIRECTORY", batchwindow->directory);
	defaults->update("KEEP", keep_original);
	defaults->update("COMPRESSOR", compressor);
	defaults->update("DECOMPRESSOR", decompressor);
	defaults->save();
}


CompressButton::CompressButton(MainWindow *window)
 : BC_BigButton(5, 6, "Compress")
{
	this->window = window;
}

CompressButton::~CompressButton()
{
}

int CompressButton::handle_event()
{
	window->set_done(0);
}



UnCompressButton::UnCompressButton(MainWindow *window)
 : BC_BigButton(110, 6, "Expand")
{
	this->window = window;
}

UnCompressButton::~UnCompressButton()
{
}

int UnCompressButton::handle_event()
{
	window->set_done(2);
}

QuitButton::QuitButton(MainWindow *window)
 : BC_BigButton(320, 6, "Cancel")
{
	this->window = window;
}

QuitButton::~QuitButton()
{
}

int QuitButton::handle_event()
{
	window->set_done(1);
}






PreferencesButton::PreferencesButton(MainWindow *window)
 : BC_BigButton(200, 6, "Preferences")
{
	this->window = window;
}

PreferencesButton::~PreferencesButton()
{
}

int PreferencesButton::handle_event()
{
	window->prefs->start();
}
