#include "preferences.h"


PreferencesThread::PreferencesThread(MainWindow *window)
 : Thread()
{
	this->window = window;
	synchronous = 0;
}

PreferencesThread::~PreferencesThread()
{
}

void PreferencesThread::run()
{
	
	PreferencesWin win(window);
	win.create_objects();
	int result = win.run_window();
	
	if(!result)
	{
		strcpy(window->compressor, win.compressor);
		strcpy(window->decompressor, win.decompressor);
		window->keep_original = win.keep_original;
	}
}






PreferencesWin::PreferencesWin(MainWindow *window)
 : BC_Window("", MEGREY, "Split 2000: Preferences", 260, 130, 0, 0)
{
	this->window = window;
}

PreferencesWin::~PreferencesWin()
{
	delete compressorbox;
	delete decompressorbox;
	delete keep_toggle;
	delete ok;
	delete cancel;
}

int PreferencesWin::create_objects()
{
	strcpy(this->compressor, window->compressor);
	strcpy(this->decompressor, window->decompressor);
	keep_original = window->keep_original;

	add_tool(new BC_Title(5, 5, "Use compressor:"));
	add_tool(compressorbox = new CompressorBox(this, compressor));
	add_tool(new BC_Title(5, 35, "Use decompressor:"));
	add_tool(decompressorbox = new DeCompressorBox(this, decompressor));
	add_tool(new BC_Title(5, 70, "Keep originals:"));
	add_tool(keep_toggle = new KeepOriginal(this, keep_original));
	add_tool(ok = new PreferencesOK(this));
	add_tool(cancel = new PreferencesCancel(this));

}


CompressorBox::CompressorBox(PreferencesWin *window, char *text)
 : BC_TextBox(150, 5, 100, text)
{
	this->window = window;
}

CompressorBox::~CompressorBox()
{
}
	
int CompressorBox::handle_event()
{
	strcpy(window->compressor, get_text());
}

DeCompressorBox::DeCompressorBox(PreferencesWin *window, char *text)
 : BC_TextBox(150, 35, 100, text)
{
	this->window = window;
}

DeCompressorBox::~DeCompressorBox()
{
}
	
int DeCompressorBox::handle_event()
{
	strcpy(window->decompressor, get_text());
}



KeepOriginal::KeepOriginal(PreferencesWin *window, int value)
 : BC_CheckBox(150, 70, 15, 15, value)
{
	this->window = window;
}

KeepOriginal::~KeepOriginal()
{
}
	
int KeepOriginal::handle_event()
{
	window->keep_original = down;
}




PreferencesOK::PreferencesOK(PreferencesWin *window)
 : BC_BigButton(5, 95, "OK")
{
	this->window = window;
}

PreferencesOK::~PreferencesOK()
{
}

int PreferencesOK::handle_event()
{
	window->set_done(0);
}

int PreferencesOK::keypress_event()
{
	if(get_keypress() == 13) { handle_event(); return  1; }
	return  0;
}



PreferencesCancel::PreferencesCancel(PreferencesWin *window)
 : BC_BigButton(100, 95, "Cancel")
{
	this->window = window;
}

PreferencesCancel::~PreferencesCancel()
{
}

int PreferencesCancel::handle_event()
{
	window->set_done(1);
}

int PreferencesCancel::keypress_event()
{
	if(get_keypress() == ESC) { handle_event(); return  1; }
	return  0;
}
