#include "progress.h"

// ================================ thread

Progress::Progress(char *title, char *text)
 : Thread(1)
{	window = new ProgressWindow(title, text); }

Progress::~Progress()
{	delete window;}

int Progress::create_objects()
{	window->create_objects(this); }

void Progress::run() {	window->run_window();}

int Progress::update_position(long position)
{	return window->update_position(position);}

int Progress::update_length(long length)
{	return window->update_length(length);}

int Progress::update_button(char *text)
{	return window->update_button(text);}

int Progress::cancelled()
{	return window->cancelled();}

int Progress::update(char *text)
{	return window->update(text);}

int Progress::stop_progress()
{
	window->set_done(0);
	Thread::end();
	Thread::join();
}

// ==================================== window

ProgressWindow::ProgressWindow(char *title, char *text)
 : BC_Window("", MEGREY, title, 500, 200, 500, 200)
{
	cancelled_ = 0;
	this->text = text;
}

ProgressWindow::~ProgressWindow()
{
	delete textarea;
	delete bar;
	delete cancel;
}

int ProgressWindow::create_objects(Progress *progress)
{
	this->progress = progress;
	
	add_tool(textarea = new BC_TextArea(10, 10, 480, 110, text));
	add_tool(bar = new BC_ProgressBar(10, 130, 480, 30, 0));
	add_tool(cancel = new ProgressCancelButton(this));
}

int ProgressWindow::update_position(long position)
{
	lock_window();
	bar->update(position);
	unlock_window();
	return cancelled_;
}

int ProgressWindow::update_length(long length)
{
	lock_window();
	bar->update_length(length);
	unlock_window();
	return cancelled_;
}

int ProgressWindow::cancelled()
{
	return cancelled_;
}


int ProgressWindow::update(char *text)
{
	lock_window();
	textarea->append_contents(text);
	unlock_window();
	return cancelled_;
}

int ProgressWindow::update_button(char *text)
{	
	static int result;
	result = 0;
	
	lock_window();
	result = cancel->update(text);
	unlock_window();
	
	return result;
}





ProgressCancelButton::ProgressCancelButton(ProgressWindow *window)
 : BC_BigButton(210, 170, "Cancel")
{ this->window = window; }

int ProgressCancelButton::handle_event()
{
	if(!window->cancelled_)
	{
		window->update("Cancelling.\n");
		window->cancelled_ = 1;
	}
// don't freeze the window the first time
	if(window->cancelled_ == 2) 
	{
		window->set_done(0);
	}
}

int ProgressCancelButton::keypress_event()
{
	if(window->get_keypress() == ESC) { handle_event(); return 1; }
	return 0;
}
