#include "splitdialog.h"

SplitDialog::SplitDialog(Defaults *defaults)
 : BC_Window("", MEGREY, "Compress", 530, 480, 0, 0)
{
	this->defaults = defaults;
}

SplitDialog::~SplitDialog()
{
	delete ok;
	delete cancel;
	delete channels_button;
#ifdef SEGMENT_SIZE
	delete segment_button;
#endif
	for(int i = 0; i < 3; i++) delete bits_items[i];
	delete bits_menu;
	delete keep;
}
	
SplitDialog::create_objects()
{
	BC_SubWindow *sub;
	char directory[1024], string[1024];
	sprintf(directory, "~");
	
	defaults->get("DIRECTORY", directory);
	channels = defaults->get("CHANNELS", 2);
	bits = defaults->get("BITS", 16);
	keep_original = defaults->get("KEEP", 0);
#ifdef SEGMENT_SIZE
	segment = defaults->get("SEGMENT", 27182);
#else
	segment = 27182;
#endif
	
	add_subwindow(batch = new BatchSubWindow(this, directory, "Files", "Batch"));
	add_subwindow(sub = new BC_SubWindow(0, 380, w, 100, MEGREY));
	sub->add_border(LTGREY, MEGREY, DKGREY);
	sub->add_tool(ok = new SplitOkButton(this));
	sub->add_tool(cancel = new SplitCancelButton(this));
	
	sprintf(string, "%d", channels);
	sub->add_tool(new BC_Title(5, 10, "Channels:"));
	sub->add_tool(channels_button = new SplitChannels(this, string));

#ifdef SEGMENT_SIZE
	sprintf(string, "%d", segment);
	sub->add_tool(new BC_Title(140, 10, "Segment size:"));
	sub->add_tool(segment_button = new SplitSegmentSize(this, string));
#endif

	sub->add_tool(new BC_Title(10, 40, "Compress:", TITLEFONT, RED));

	sprintf(string, "%d", bits);
	sub->add_tool(new BC_Title(330, 10, "Bits:"));
	sub->add_tool(bits_menu = new SplitBits(this, string));
	bits_menu->add_item(bits_items[0] = new SplitBitsItem(bits_menu, "8"));
	bits_menu->add_item(bits_items[1] = new SplitBitsItem(bits_menu, "16"));
	bits_menu->add_item(bits_items[2] = new SplitBitsItem(bits_menu, "24"));

	sub->add_tool(new BC_Title(200, 10, "Keep original:"));
	sub->add_tool(keep = new SplitKeepOriginal(this, keep_original));
}

SplitDialog::resize_event(int w, int h)
{
	this->w = w; this->h = h;
}

SplitDialog::doit()
{
	defaults->update("DIRECTORY", batch->directory);
	defaults->update("CHANNELS", channels);
	defaults->update("BITS", bits);
	defaults->update("KEEP", keep_original);
#ifdef SEGMENT_SIZE
	defaults->update("SEGMENT", segment);
#endif
	set_done(0);
}

SplitOkButton::SplitOkButton(SplitDialog *dialog)
 : BC_BigButton(10, 70, "Do it")
{
	this->dialog = dialog;
}

SplitOkButton::~SplitOkButton()
{
}

SplitOkButton::handle_event()
{
	dialog->doit();
}

SplitOkButton::keypress_event()
{
	return 0;
}

SplitCancelButton::SplitCancelButton(SplitDialog *dialog)
 : BC_BigButton(320, 70, "Cancel")
{
	this->dialog = dialog;
}

SplitCancelButton::~SplitCancelButton()
{
}
	
SplitCancelButton::handle_event()
{
	dialog->set_done(1);
}

SplitCancelButton::keypress_event()
{
	return 0;
}

SplitChannels::SplitChannels(SplitDialog *dialog, char *text)
 : BC_TextBox(80, 5, 50, text)
{
	this->dialog = dialog;
}

SplitChannels::~SplitChannels()
{
}
	
SplitChannels::handle_event()
{
	dialog->channels = atol(text);
}

SplitSegmentSize::SplitSegmentSize(SplitDialog *dialog, char *text)
 : BC_TextBox(240, 5, 75, text)
{
	this->dialog = dialog;
}

SplitSegmentSize::~SplitSegmentSize()
{
}
	
SplitSegmentSize::handle_event()
{
	dialog->segment = atol(text);
}

SplitBits::SplitBits(SplitDialog *dialog, char *text)
 : BC_PopupMenu(370, 5, 50, text)
{
	this->dialog = dialog;
}

SplitBits::~SplitBits()
{
}
	
SplitBits::handle_event()
{
	dialog->bits = atol(text);
}

SplitKeepOriginal::SplitKeepOriginal(SplitDialog *dialog, int value)
 : BC_CheckBox(300, 10, 15, 15, value)
{
	this->dialog = dialog;
}

SplitKeepOriginal::~SplitKeepOriginal()
{
}
	
SplitKeepOriginal::handle_event()
{
	dialog->keep_original = down;
}

SplitBitsItem::SplitBitsItem(SplitBits *bits, char *text)
 : BC_PopupItem(text)
{
	this->bits = bits;
}

SplitBitsItem::~SplitBitsItem()
{
}
	
SplitBitsItem::handle_event()
{
	menu->update(text);
	menu->handle_event();
}
