#include "filesystem.h"
#include "splitengine.h"

#include <unistd.h>

// algorithm >=16 bits:
// [1st byte[channel1 segment][channel2 segment][channel1 segment][channel2 segment]][2nd byte[channel1 segment][channel2 segment]...

// algorithm 8 bits:
// [1st 4 bits[channel1 segment][channel2 segment][channel1 segment]...][last 4 bits[channel1 segment][channel2 segment]..]

SplitEngine::SplitEngine(int channels, 
			int keep_original, 
			int bits, 
			int segment, 
			ArrayList <BatchItem*> *batch, 
			char *compressor,
			char *output_directory)
{
	this->channels = channels;
	this->keep_original = keep_original;
	this->bits = bits;
	this->segment = segment;
	this->batch = batch;
	this->compressor = compressor;
	strcpy(this->output_directory, output_directory);
}

SplitEngine::~SplitEngine()
{
}

int SplitEngine::put_number(long number, FILE *out)
{
	fputc((number & 0xFF000000) >> 24, out); 
	fputc((number & 0xFF0000) >> 16, out); 
	fputc((number & 0xFF00) >> 8, out); 
	fputc((number & 0xFF), out); 
}

int SplitEngine::update_user(char *string)
{        // stdout is the output file so use stderr 
	if(interactive) progress->update(string);
	else 
	fprintf(stderr, "%s", string);
}

int SplitEngine::run(int interactive)
{
// set up display
	char text[65535];
	char string[1024];
	char filename_in_text[1024], filename_out_text[1024];

	sprintf(text, "");
	this->interactive = interactive;

	if(interactive)
	{
		progress = new Progress("Compressor", text);
		progress->create_objects();
		progress->start();
	}

	register int bytes = bits / 8;
	int result = 0;
	register int frame = bytes * channels;

// get input buffer length
	long buffer_in_segment = segment * bytes * channels;
	if(bits == 8 && channels == 1)
	{
		segment /= 2;       // segment must be divisible by nibbles
		segment *= 2;
		buffer_in_segment = segment * 2 * channels;
	}

	long buffer_in_total = 0;

// Egcs math error
//	while(buffer_in_total < 1000000) buffer_in_total += buffer_in_segment;
	
	buffer_in_total = (int)(1000000 / buffer_in_segment) * buffer_in_segment;

	
//printf("SplitEngine %ld %ld\n", buffer_in_total, buffer_in_segment);
	char *buffer_in = new char[buffer_in_total];
	long buffer_out_total = buffer_in_total / bytes + 1;
// one nibble for every byte
	if(bits == 8 && channels == 1) buffer_out_total = buffer_in_total / 2 + 1;
	char *buffer_out = new char[buffer_out_total];
	FileSystem fs;

	for(int i = 0; i < batch->total && result != 1; i++)
	{
		FILE *in, *out, *stdout_alias;
		char filename_in[1024], filename_out[1024], exec_script[1024];

// ================================= create output filenames and scripts
		strcpy(filename_in, batch->values[i]->path);
		fs.extract_name(filename_in_text, filename_in);
		fs.extract_name(filename_out_text, filename_in);
		strcat(filename_out_text, SUFFIX);

		fs.join_names(filename_out, output_directory, filename_out_text);


		sprintf(exec_script, "%s", compressor);

// =============================== test existence of output file
		if(!result && (in = fopen(filename_out, "rb")))
		{
			sprintf(string, "%s already exists.  Skipping.\n", filename_out);
			update_user(string);
			result = 2;
			fclose(in);
		}
	
// open the input file		
		if(!result && !(in = fopen(filename_in, "rb")))
		{
			sprintf(string, "Couldn't open %s.\n", filename_in);
			update_user(string);
			result = 1;
		}
		
// redirect the stdout to stdout_alias
		if(!result && !(stdout_alias = fopen(filename_out, "wb")))
		{
			sprintf(string, "Couldn't open %s.\n", filename_out);
			update_user(string);
			fclose(in);
			result = 1;
		}
		if(!result) dup2(fileno(stdout_alias), 1);
		
// open the output file
		if(!result && !(out = popen(exec_script, "wb")))
		{
			sprintf(string, "Couldn't fork compressor.\n");
			update_user(string);
			fclose(in);
			fclose(stdout_alias);
			result = 1;
		}
		
		if(!result){
// compute file lengths
			fseek(in, 0, SEEK_END);
			long length_in = ftell(in);           // total length
			long length_out = length_in / bytes;  // length of a byte strip
			long length_byte_strip = 0;           // length of byte strip after segmentation
			
			while(length_byte_strip < length_out) length_byte_strip += segment * channels;
			if(bits == 8 && channels == 1)
			{         // 8 bits use nibbles
				length_out /= 2;
				length_byte_strip = 0;
				while(length_byte_strip < length_out) length_byte_strip += segment * channels;
			}
// store header

			put_number(7374, out); // magic number
			put_number(segment, out); // segment size
			put_number(channels, out); // channels
			put_number(bits, out); // bits
			put_number(length_byte_strip, out);     // length of a byte strip
			put_number(length_in, out);       // store real length of file
			put_number(0, out);       // padding
			put_number(0, out);       // padding
			put_number(0, out);       // padding
			put_number(0, out);       // padding
			put_number(0, out);       // padding
			
// reset length_out for counter
			length_out = 0;
			
// display some information
			sprintf(string, "%s -> %s\n", filename_in_text, filename_out_text);
			update_user(string);
			
#ifdef SEGMENT_SIZE
			sprintf(string, "Settings: %d channels %d bits %d segment size\n", channels, bits, segment);
#else
			sprintf(string, "Settings: %d channels %d bits\n", channels, bits);
#endif
			update_user(string);
			sprintf(string, "Using: %s\n", compressor);
			update_user(string);
			if(keep_original) sprintf(string, "Keeping original.\n");
			else sprintf(string, "Deleting original.\n");
			update_user(string);

			if(interactive) progress->update_length(length_in);

// ======================================== 8 bits
			if(bits == 8 && channels == 1)
			{
				for(int input_nibble = 0; input_nibble < 2; input_nibble++)
				{
					fseek(in, 0, SEEK_SET);
					long buffer_in_length = buffer_in_total;

					while(ftell(in) < length_in && !result)
					{
						if(length_in - ftell(in) < buffer_in_length)
						{
							buffer_in_length = length_in - ftell(in);
							for(int j = 0; j < buffer_in_total;) buffer_in[j++] = 0;
						}

	// read a buffer length					
						if(!fread(buffer_in, buffer_in_length, 1, in))
						{
							result = 1;
							sprintf(string, "Read failed.\n");
							update_user(string);
						}
						else
						{
	// set up the channel pointers for the input buffer
							register long channel_pointer_in[channels];
							for(int j = 0; j < channels; j++)
							{
								channel_pointer_in[j] = j;
							}

	// set up the channel pointers for the output buffer
							register long channel_pointer_out[channels];
							for(int j = 0; j < channels; j++)
							{
								channel_pointer_out[j] = j * segment;
							}

	// ================ split the input buffer into the output buffer
							register int segment_pointer;
							register int channel;
							register char nibble;

							while(channel_pointer_in[0] < buffer_in_length)
							{
	// fill segments for all channels simultaneously
								if(input_nibble == 0)
								{
// ========================== high input nibble
									for(segment_pointer = 0; segment_pointer < segment; segment_pointer++)
									{
		// iterate through each channel for each segment
										for(channel = 0; channel < channels; channel++)
										{
		// put high nibble in segment
											nibble = buffer_in[channel_pointer_in[channel]];
											nibble &= 0xF0;
											buffer_out[channel_pointer_out[channel]] = nibble;
											channel_pointer_in[channel] += frame;

		// put low nibble in segment
											nibble = buffer_in[channel_pointer_in[channel]];
											nibble >>= 4;
											nibble &= 0xF;
											buffer_out[channel_pointer_out[channel]] += nibble;
											channel_pointer_in[channel] += frame;

											channel_pointer_out[channel]++;
										}
									}
		// realign channel pointers for next segment
									for(channel = 0; channel < channels; channel++)
									{      
										channel_pointer_out[channel] += segment * channels - segment;
									}
								}
								else
								{
// ================================== low input nibble
									for(segment_pointer = 0; segment_pointer < segment; segment_pointer++)
									{
		// iterate through each channel for each segment
										for(channel = 0; channel < channels; channel++)
										{
		// put high nibble in segment
											nibble = buffer_in[channel_pointer_in[channel]];
											nibble <<= 4;
											nibble &= 0xF0;
											buffer_out[channel_pointer_out[channel]] = nibble;
											channel_pointer_in[channel] += frame;

		// put low nibble in segment
											nibble = buffer_in[channel_pointer_in[channel]];
											nibble &= 0x0F;
											buffer_out[channel_pointer_out[channel]] += nibble;
											channel_pointer_in[channel] += frame;

											channel_pointer_out[channel]++;
										}
									}
		// realign channel pointers for next segment
									for(channel = 0; channel < channels; channel++)
									{      
										channel_pointer_out[channel] += segment * channels - segment;
									}
								}
							}

//fprintf(stderr, "%d %d\n", channel_pointer_in[0], channel_pointer_out[0]);
							if(!fwrite(buffer_out, channel_pointer_out[0], 1, out))
							{
								sprintf(string, "Write failed.\n");
								update_user(string);
								result = 1;
							}
	// increase output length
							length_out += channel_pointer_out[0];

// update progress
// fake real length
							long total_result = length_out;
							if(total_result > length_in) total_result = length_in;
							if(!interactive)
							{
								fprintf(stderr, "%ld bytes split\r", total_result);
								fflush(stderr);
							}
							else
							{
								if(progress->update_position(total_result)) result = 1;
							}
						}
					}
				}
			}
			else
// ======================================== >=16 bits
			{
// write each byte from the file
				for(int byte = 0; byte < bytes && !result; byte++)
				{
					fseek(in, byte, SEEK_SET);
					long buffer_in_length = buffer_in_total;

					while(ftell(in) < length_in && !result)
					{
						if(length_in - ftell(in) < buffer_in_length)
						{
							buffer_in_length = length_in - ftell(in);
							for(int j = 0; j < buffer_in_total;) buffer_in[j++] = 0;
						}

	// read a buffer length					
						if(!fread(buffer_in, buffer_in_length, 1, in))
						{
							result = 1;
							sprintf(string, "Read failed.\n");
							update_user(string);
						}
						else
						{
	// set up the channel pointers for the input buffer
							register long channel_pointer_in[channels];
							for(int j = 0; j < channels; j++)
							{
								channel_pointer_in[j] = j * bytes;
							}

	// set up the channel pointers for the output buffer
							register long channel_pointer_out[channels];
							for(int j = 0; j < channels; j++)
							{
								channel_pointer_out[j] = j * segment;
							}

	// split the input buffer into the output buffer
							register int segment_pointer;
							register int channel;

							while(channel_pointer_in[0] < buffer_in_length)
							{
	// fill segments for all channels simultaneously
								for(segment_pointer = 0; segment_pointer < segment; segment_pointer++)
								{
	// iterate through each channel for each segment
									for(channel = 0; channel < channels; channel++)
									{
										buffer_out[channel_pointer_out[channel]] = buffer_in[channel_pointer_in[channel]];
										channel_pointer_out[channel]++;
										channel_pointer_in[channel] += frame;
									}
								}
	// realign channel pointers for next segment
								for(channel = 0; channel < channels; channel++)
								{      
									channel_pointer_out[channel] += segment * channels - segment;
								}
							}

							if(!fwrite(buffer_out, channel_pointer_out[0], 1, out))
							{
								sprintf(string, "Write failed.\n");
								update_user(string);
								result = 1;
							}
	// increase output length
							length_out += channel_pointer_out[0];

	// update progress
// fake real length
							long total_result = length_out;
							if(total_result > length_in) total_result = length_in;
							if(!interactive)
							{
								fprintf(stderr, "%ld bytes split\r", total_result);
								fflush(stderr);
							}
							else
							{
								if(progress->update_position(total_result)) result = 1;
							}
						}
					}
				}
			}// end >=16 bits

			fclose(in);
			fclose(out);
			fclose(stdout_alias);

// remove original or output if error
			if(!keep_original && !result) remove(filename_in);
						
			if(result == 1) remove(filename_out);
			if(!result)
			{
				out = fopen(filename_out, "rb");
				fseek(out, 0, SEEK_END);
				length_out = ftell(out);
				
				int ratio = (int)(100 * (float)(length_in - length_out) / length_in);
				
// add eoln for non interactive
				if(!interactive) fprintf(stderr, "\n");
				if(interactive) progress->update_position(length_in);
				sprintf(string, "Compressed by %d%%\n\n", ratio);
				update_user(string);
			}
		}
	}
	
	delete buffer_in;
	delete buffer_out;
	
	if(interactive) 
	{
		//progress->end(); // prevents core dump
		progress->update("Done\n");
		progress->update_button("OK");
		progress->window->cancelled_ = 2;
		//progress->run(); 
		progress->join();      // wait for OK
		delete progress; 
	}
}
